/* let DEFAULT_HASH =
  "0x175adf5fc058830a6319b8238ecc911db6e1b8dd40965629b5f0c5bee655598c"; */

let DEFAULT_HASH;
let hash;
let seed = 0;
let anchors = [];
let selectedPalette = [];
let lineThickness;

let capturing = false;
let capturer;

let firstDrawComplete = false;

function generateRandomHash() {
  const chars = "0123456789abcdef";
  let hash = "0x";
  for (let i = 0; i < 64; i++) {
    hash += chars[Math.floor(Math.random() * chars.length)];
  }
  return hash;
}

DEFAULT_HASH = generateRandomHash();

function setup() {
  canvasSize = min(windowWidth, windowHeight);
  createCanvas(canvasSize, canvasSize);
  lineThickness = calculateLineThickness();

  const params = getURLParams();
  hash = params.hash || DEFAULT_HASH;
  seed = hashToSeed(hash);
  randomSeed(seed);
  noiseSeed(seed);
  console.log(hash, seed);

  randDirectLine = int(random(2));
  randSizeCan = random(50, 70);
  frameRate(5);

  selectedPalette = choosePalette();

  createTopography(random(1, 2), random(8, 30));
  capturer = new CCapture({
    format: "gif",
    name: "rivages",
    workersPath: "lib/",
    framerate: 5,
    timeLimit: 2,
    verbose: true,
    StartTime: 1,
  });
}

function choosePalette() {
  const palettes = [
    [
      { r: 202, g: 255, b: 10 }, 
      { r: 0, g: 0, b: 0 },
      { r: 202, g: 255, b: 10 },
    ],
    [
      { r: 0, g: 0, b: 0 },
      { r: 202, g: 255, b: 10 }, 
      { r: 0, g: 0, b: 0 },
    ],
    [
      { r: 0, g: 0, b: 0 },
      { r: 90, g: 250, b: 90 },
    ],

    [
      { r: 10, g: 250, b: 255 },
      { r: 0, g: 0, b: 0 },
      { r: 10, g: 250, b: 255 },
    ],
    [
      { r: 0, g: 0, b: 0 },
      { r: 0, g: 0, b: 0 },
      { r: 0, g: 0, b: 0 },
    ],

    [
      { r: 0, g: 255, b: 210 }, 
      { r: 0, g: 0, b: 0 }, 
      { r: 0, g: 255, b: 210 }, 
    ],
    [
      { r: 0, g: 0, b: 0 }, 
      { r: 0, g: 255, b: 210 }, 

      { r: 0, g: 0, b: 0 }, 
    ],

    [
      { r: 0, g: 120, b: 255 }, 
      { r: 0, g: 0, b: 0 }, 
      { r: 0, g: 120, b: 255 }, 
    ],
    [
      { r: 0, g: 0, b: 0 }, 
      { r: 0, g: 120, b: 255 }, 
      { r: 0, g: 0, b: 0 }, 
    ],

    [
      { r: 255, g: 0, b: 255 },
      { r: 0, g: 0, b: 0 },
    ],

    [
      { r: 255, g: 255, b: 0 },
      { r: 0, g: 0, b: 0 },
      { r: 255, g: 255, b: 0 },
    ],
    [
      { r: 0, g: 0, b: 0 },
      { r: 255, g: 255, b: 0 }, 
      { r: 0, g: 0, b: 0 },
    ],
    [
      { r: 85, g: 60, b: 250 }, 
      { r: 0, g: 0, b: 0 },
      { r: 85, g: 60, b: 250 },
    ],
    [
      { r: 0, g: 0, b: 0 },
      { r: 85, g: 60, b: 250 }, 
      { r: 0, g: 0, b: 0 },
    ],

    [
      { r: 0, g: 0, b: 0 },
      { r: 85, g: 60, b: 250 }, 
      { r: 0, g: 0, b: 0 },
    ],
    [
      { r: 85, g: 60, b: 250 }, 
      { r: 0, g: 0, b: 0 },
      { r: 85, g: 60, b: 250 }, 
    ],
  ];

  return random(palettes);
}

function getColorFromPaletteBasedOnPosition(anchor) {
  let canvasCenterX = width / 2;
  let canvasCenterY = height / 2;
  let distToCenter = dist(anchor.x, anchor.y, canvasCenterX, canvasCenterY);
  let colorIndex = floor(
    map(distToCenter, 0, max(width, height) / 2, 0, selectedPalette.length)
  );

  colorIndex = constrain(colorIndex, 0, selectedPalette.length - 1);
  let selectedColor = selectedPalette[colorIndex];
  let alpha = 235;
  return color(selectedColor.r, selectedColor.g, selectedColor.b, alpha);
}

function createTopography(numLayers, pointsPerLayer) {
  let baseHeight = height;
  for (let i = 0; i < numLayers; i++) {
    let layerHeight = map(i, 0, numLayers, height * 0.3, height * 0.4);
    let yOffset = baseHeight - layerHeight;
    let radius = width / 3;
    let angleStep = TWO_PI / pointsPerLayer;
    let xOffset = random(-width * 0.3, width * 0.3);
    let yOffsetNoise = random(-height * 0.05, height * 0.05);
    beginShape();
    for (let j = 0; j < pointsPerLayer; j++) {
      let angle = angleStep * j;
      let x =
        width / 2 +
        xOffset +
        radius * cos(angle) +
        random(-radius * 0.3, radius * 0.3);
      let y =
        yOffset +
        radius * sin(angle) +
        yOffsetNoise +
        random(-radius * 0.8, radius * 0.8);

    
      x = constrain(x, 0, width);
      y = constrain(y, 0, height);

      noStroke();
      vertex(x, y);
      anchors.push({
        x: x,
        y: y,
        lines: int(random(1, 5)),
      });
    }
    endShape(CLOSE);
  }
}

function calculateLineThickness() {
  return max(1, canvasSize / 200);
}

function draw() {
  background(255);
  let size = canvasSize / randSizeCan;
  strokeWeight(lineThickness);

  for (let y = 0; y < height; y += size) {
    for (let x = 0; x < width; x += size) {
      let closestAnchor = getClosestAnchor(x, y);
      if (closestAnchor) {
        let lineThickness = random(2, 4);
        strokeWeight(lineThickness);
        if (random(1) > randDirectLine) {
          drawVerticalLines(x, y, size, closestAnchor.lines);
        } else {
          drawHorizontalLines(x, y, size, closestAnchor.lines);
        }
      }
    }
  }

  if (!firstDrawComplete) {
    firstDrawComplete = true;
    postMessageKoda();
  }

  if (capturing) {
    capturer.capture(canvas);
  }
}

function keyPressed() {
  if (key === "s" || key === "S") {
    saveCanvas("rivages", "jpg");
  }

  if (key === "g" || key === "G") {
    if (!capturing) {
      capturer.start();
      capturing = true;
    } else {
      //capturer.stop();
      capturer.save();
      capturing = false;
    }
  }
}

function getClosestAnchor(x, y) {
  let closestAnchor = null;
  let closestDist = Infinity;
  for (let anchor of anchors) {
    let d = dist(x, y, anchor.x, anchor.y);
    if (d < closestDist) {
      closestDist = d;
      closestAnchor = anchor;
    }
  }
  return closestAnchor;
}

function drawVerticalLines(x, y, size, lines) {
  let lineSpacing = size / lines;
  let closestAnchor = getClosestAnchor(x, y);
  stroke(getColorFromPaletteBasedOnPosition(closestAnchor));
  for (let i = 0; i < lines; i++) {
    line(x + i * lineSpacing, y, x + i * lineSpacing, y + size);
  }
}

function drawHorizontalLines(x, y, size, lines) {
  let lineSpacing = size / lines;
  let closestAnchor = getClosestAnchor(x, y);
  stroke(getColorFromPaletteBasedOnPosition(closestAnchor));
  for (let i = 0; i < lines; i++) {
    line(x, y + i * lineSpacing, x + size, y + i * lineSpacing);
  }
}

function windowResized() {
  canvasSize = min(windowWidth, windowHeight);
  resizeCanvas(canvasSize, canvasSize);
  lineThickness = calculateLineThickness();
}

function hashToSeed(hash) {
  let _seed = 0;
  for (let hl = 0; hl < 60; hl = hl + 12) {
    _seed += parseInt(hash.substring(hl, hl + 12), 16);
  }
  return _seed;
}

function postMessageKoda() {
  const canvas = document.querySelector("canvas");
  const message = {
    id: Date.now(),
    type: "kodahash/render/completed",
    payload: {
      hash: hash,
      type: "image/png",
      image: canvas ? canvas.toDataURL("image/png") : null,
      search: location.search,
      attributes: [],
    },
  };
  window.parent.postMessage(message, "*");
}
